==> symbol-processing-aa-embeddable-2.1.0-1.0.29-sources.jar: org/jetbrains/kotlin/backend/jvm/serialization/JvmIrSerializerSession.kt <==
/*
 * Copyright 2010-2021 JetBrains s.r.o. and Kotlin Programming Language contributors.
 * Use of this source code is governed by the Apache 2.0 license that can be found in the license/LICENSE.txt file.
 */

package ksp.org.jetbrains.kotlin.backend.jvm.serialization

import ksp.org.jetbrains.kotlin.backend.common.serialization.DeclarationTable
import ksp.org.jetbrains.kotlin.backend.common.serialization.IrFileSerializer
import ksp.org.jetbrains.kotlin.backend.common.serialization.IrSerializationSettings
import ksp.org.jetbrains.kotlin.backend.jvm.serialization.proto.JvmIr
import ksp.org.jetbrains.kotlin.config.JvmSerializeIrMode
import ksp.org.jetbrains.kotlin.config.LanguageVersionSettings
import ksp.org.jetbrains.kotlin.ir.IrElement
import ksp.org.jetbrains.kotlin.ir.declarations.*
import ksp.org.jetbrains.kotlin.ir.visitors.IrElementVisitor
import ksp.org.jetbrains.kotlin.name.FqName
import ksp.org.jetbrains.kotlin.synthetic.isVisibleOutside

class JvmIrSerializerSession(
    declarationTable: DeclarationTable.Default,
    private val mode: JvmSerializeIrMode,
    private val fileClassFqName: FqName,
    languageVersionSettings: LanguageVersionSettings,
) : IrFileSerializer(
    IrSerializationSettings(
        languageVersionSettings = languageVersionSettings,
        bodiesOnlyForInlines = mode == JvmSerializeIrMode.INLINE,
    ),
    declarationTable,
) {
    init {
        assert(mode != JvmSerializeIrMode.NONE)
    }

    // Usage protocol: construct an instance, call only one of `serializeIrFile()` and `serializeTopLevelClass()` only once.

    fun serializeJvmIrFile(irFile: IrFile): JvmIr.ClassOrFile? {
        var anySaved = false
        val proto = JvmIr.ClassOrFile.newBuilder()

        inFile(irFile) {
            irFile.declarations.filter { it !is IrClass }.forEach { topDeclaration ->
                forEveryDeclarationToSerialize(topDeclaration, mode) { declaration ->
                    proto.addDeclaration(serializeDeclaration(declaration))
                    anySaved = true
                }
            }
        }
        if (!anySaved) return null

        serializeAuxTables(proto)
        proto.fileFacadeFqName = fileClassFqName.asString()

        return proto.build()
    }

    fun serializeTopLevelClass(irClass: IrClass): JvmIr.ClassOrFile? {
        val proto = JvmIr.ClassOrFile.newBuilder()
        inFile(irClass.parent as IrFile) {
            forEveryDeclarationToSerialize(irClass, mode) { declaration ->
                proto.addDeclaration(serializeDeclaration(declaration))
            }
        }
        serializeAuxTables(proto)
        proto.fileFacadeFqName = fileClassFqName.asString()

        return proto.build()
    }

    private fun serializeAuxTables(proto: JvmIr.ClassOrFile.Builder) {
        protoTypeArray.protoTypes.forEach(proto::addType)
        protoIdSignatureArray.forEach(proto::addSignature)
        protoStringArray.forEach(proto::addString)
        protoBodyArray.forEach { proto.addBody(it.toProto()) }
        protoDebugInfoArray.forEach(proto::addDebugInfo)
    }

    fun XStatementOrExpression.toProto(): JvmIr.XStatementOrExpression = when (this) {
        is XStatementOrExpression.XStatement -> JvmIr.XStatementOrExpression.newBuilder().setStatement(toProtoStatement()).build()
        is XStatementOrExpression.XExpression -> JvmIr.XStatementOrExpression.newBuilder().setExpression(toProtoExpression()).build()
    }
}

private fun forEveryDeclarationToSerialize(topDeclaration: IrDeclaration, mode: JvmSerializeIrMode, action: (IrDeclaration) -> Unit) {
    when (mode) {
        JvmSerializeIrMode.NONE -> error("should not even be called with serialization mode NONE")
        JvmSerializeIrMode.ALL -> action(topDeclaration)
        JvmSerializeIrMode.INLINE ->
            topDeclaration.accept(ForVisibleInlineFunctionsVisitor, action)
    }
}

private object ForVisibleInlineFunctionsVisitor : IrElementVisitor<Unit, (IrDeclaration) -> Unit> {
    override fun visitElement(element: IrElement, data: (IrDeclaration) -> Unit) {
        error("Visitor only for nonlocal declarations")
    }

    override fun visitDeclaration(declaration: IrDeclarationBase, data: (IrDeclaration) -> Unit) {
        return
    }

    override fun visitClass(declaration: IrClass, data: (IrDeclaration) -> Unit) {
        if (!declaration.visibility.isVisibleOutside()) return
        for (child in declaration.declarations) {
            child.accept(this, data)
        }
    }

    override fun visitProperty(declaration: IrProperty, data: (IrDeclaration) -> Unit) {
        if (!declaration.visibility.isVisibleOutside()) return
        declaration.getter?.accept(this, data)
        declaration.setter?.accept(this, data)
    }

    override fun visitSimpleFunction(declaration: IrSimpleFunction, data: (IrDeclaration) -> Unit) {
        val action = data
        if (declaration.visibility.isVisibleOutside() &&
            declaration.isInline &&
            !declaration.isFakeOverride
        ) {
            action(declaration)
        }
    }
}

==> symbol-processing-aa-embeddable-2.2.0-2.0.2-sources.jar: org/jetbrains/kotlin/backend/jvm/serialization/JvmIrSerializerSession.kt <==
/*
 * Copyright 2010-2021 JetBrains s.r.o. and Kotlin Programming Language contributors.
 * Use of this source code is governed by the Apache 2.0 license that can be found in the license/LICENSE.txt file.
 */

package ksp.org.jetbrains.kotlin.backend.jvm.serialization

import ksp.org.jetbrains.kotlin.backend.common.serialization.DeclarationTable
import ksp.org.jetbrains.kotlin.backend.common.serialization.IrFileSerializer
import ksp.org.jetbrains.kotlin.backend.common.serialization.IrSerializationSettings
import ksp.org.jetbrains.kotlin.backend.jvm.serialization.proto.JvmIr
import ksp.org.jetbrains.kotlin.config.CompilerConfiguration
import ksp.org.jetbrains.kotlin.config.JvmSerializeIrMode
import ksp.org.jetbrains.kotlin.ir.IrElement
import ksp.org.jetbrains.kotlin.ir.declarations.*
import ksp.org.jetbrains.kotlin.ir.visitors.IrVisitor
import ksp.org.jetbrains.kotlin.name.FqName
import ksp.org.jetbrains.kotlin.synthetic.isVisibleOutside

class JvmIrSerializerSession(
    declarationTable: DeclarationTable.Default,
    private val mode: JvmSerializeIrMode,
    private val fileClassFqName: FqName,
    configuration: CompilerConfiguration,
) : IrFileSerializer(
    IrSerializationSettings(
        configuration = configuration,
        bodiesOnlyForInlines = mode == JvmSerializeIrMode.INLINE,
        reuseExistingSignaturesForSymbols = false,
    ),
    declarationTable,
) {
    init {
        assert(mode != JvmSerializeIrMode.NONE)
    }

    // Usage protocol: construct an instance, call only one of `serializeIrFile()` and `serializeTopLevelClass()` only once.

    fun serializeJvmIrFile(irFile: IrFile): JvmIr.ClassOrFile? {
        var anySaved = false
        val proto = JvmIr.ClassOrFile.newBuilder()

        inFile(irFile) {
            irFile.declarations.filter { it !is IrClass }.forEach { topDeclaration ->
                forEveryDeclarationToSerialize(topDeclaration, mode) { declaration ->
                    proto.addDeclaration(serializeDeclaration(declaration))
                    anySaved = true
                }
            }
        }
        if (!anySaved) return null

        serializeAuxTables(proto)
        proto.fileFacadeFqName = fileClassFqName.asString()
        protoIrFileEntryArray.forEach(proto::addFileEntry)

        return proto.build()
    }

    fun serializeTopLevelClass(irClass: IrClass): JvmIr.ClassOrFile? {
        val proto = JvmIr.ClassOrFile.newBuilder()
        inFile(irClass.parent as IrFile) {
            forEveryDeclarationToSerialize(irClass, mode) { declaration ->
                proto.addDeclaration(serializeDeclaration(declaration))
            }
        }
        serializeAuxTables(proto)
        proto.fileFacadeFqName = fileClassFqName.asString()
        protoIrFileEntryArray.forEach(proto::addFileEntry)

        return proto.build()
    }

    private fun serializeAuxTables(proto: JvmIr.ClassOrFile.Builder) {
        protoTypeArray.protoTypes.forEach(proto::addType)
        protoIdSignatureArray.forEach(proto::addSignature)
        protoStringArray.forEach(proto::addString)
        protoBodyArray.forEach { proto.addBody(it.toProto()) }
        protoDebugInfoArray.forEach(proto::addDebugInfo)
    }

    fun XStatementOrExpression.toProto(): JvmIr.XStatementOrExpression = when (this) {
        is XStatementOrExpression.XStatement -> JvmIr.XStatementOrExpression.newBuilder().setStatement(toProtoStatement()).build()
        is XStatementOrExpression.XExpression -> JvmIr.XStatementOrExpression.newBuilder().setExpression(toProtoExpression()).build()
    }
}

private fun forEveryDeclarationToSerialize(topDeclaration: IrDeclaration, mode: JvmSerializeIrMode, action: (IrDeclaration) -> Unit) {
    when (mode) {
        JvmSerializeIrMode.NONE -> error("should not even be called with serialization mode NONE")
        JvmSerializeIrMode.ALL -> action(topDeclaration)
        JvmSerializeIrMode.INLINE ->
            topDeclaration.accept(ForVisibleInlineFunctionsVisitor, action)
    }
}

private object ForVisibleInlineFunctionsVisitor : IrVisitor<Unit, (IrDeclaration) -> Unit>() {
    override fun visitElement(element: IrElement, data: (IrDeclaration) -> Unit) {
        error("Visitor only for nonlocal declarations")
    }

    override fun visitDeclaration(declaration: IrDeclarationBase, data: (IrDeclaration) -> Unit) {
        return
    }

    override fun visitClass(declaration: IrClass, data: (IrDeclaration) -> Unit) {
        if (!declaration.visibility.isVisibleOutside()) return
        for (child in declaration.declarations) {
            child.accept(this, data)
        }
    }

    override fun visitProperty(declaration: IrProperty, data: (IrDeclaration) -> Unit) {
        if (!declaration.visibility.isVisibleOutside()) return
        declaration.getter?.accept(this, data)
        declaration.setter?.accept(this, data)
    }

    override fun visitSimpleFunction(declaration: IrSimpleFunction, data: (IrDeclaration) -> Unit) {
        val action = data
        if (declaration.visibility.isVisibleOutside() &&
            declaration.isInline &&
            !declaration.isFakeOverride
        ) {
            action(declaration)
        }
    }
}
